#region Header
/*----------------------------------------------------------------------------------------------------------------------
 * Description : FileDrop component, this component makes it easy to drag and drop files from the Windows Explorer 
 * to your .NET application. Just link a control or a form to this FileDrop component and 
 * set the list of allowed file extensions.
 
 * File : FileDrop.cs
 * Namespaces : ScipBe.Controls.WinForms
 * Classes : FileDrop, FilesDroppedEventArgs, FileExtensionList, FileExtension, AllowedFileExtensionsConverter
 
 * Dates : November 2006 - December 2008
 * Version : 1.1

 * Author : Stefan Cruysberghs
 * Website : http://www.scip.be
------------------------------------------------------------------------------------------------------------------------
Version: MPL 1.1
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/

 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the License.
 
 * The Original Code is ScipBe.Controls.WinForms
 
 * The Initial Developer of the Original Code is Stefan Cruysberghs, http://www.scip.be
 * All Rights Reserved.
------------------------------------------------------------------------------------------------------------------------
History
 * Version 1.0 (November 2006) : First version
------------------------------------------------------------------------------------------------------------------------*/
#endregion

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.IO;
using System.Text;
using System.Windows.Forms;

internal class ResourceFinder
{
}

namespace ScipBe.Controls.WinForms
{
  #region AllowedFileExtensionsConverter class
  /// <summary>
  /// TypeConverter for design-time support
  /// </summary> 
  public class AllowedFileExtensionsConverter : TypeConverter
  {
    public override object ConvertTo(ITypeDescriptorContext context,
     System.Globalization.CultureInfo culture,
     object value,
     Type destinationType)
    {
      return (value as FileExtensionList).GetList();
    }
  }
  #endregion

  #region FilesDroppedEventArgs class
  /// <summary>
  /// Event parameters when dropping files
  /// </summary>   
  public class FilesDroppedEventArgs
  {             
    public FilesDroppedEventArgs(List<string> fileNames, Point dropPoint, int totalFilesCount, int acceptedFilesCount)
    {
      this.fileNames = fileNames;
      this.dropPoint = dropPoint;
      this.totalFilesCount = totalFilesCount;
      this.acceptedFilesCount = acceptedFilesCount;
    }

    private readonly List<string> fileNames;
    /// <summary>
    /// List of filenames which have been accepted
    /// </summary> 
    public List<string> FileNames
    {
      get { return this.fileNames; }
    }

    private readonly Point dropPoint;
    /// <summary>
    /// Coordinates where files have been dropped on linked control/form
    /// </summary> 
    public Point DropPoint
    {
      get { return this.dropPoint; }
    }

    private readonly int totalFilesCount;
    /// <summary>
    /// Total number of files which have been dropped
    /// </summary> 
    public int TotalFilesCount
    {
      get { return this.totalFilesCount; }
    }

    private readonly int acceptedFilesCount;
    /// <summary>
    /// Number of files which have been accepted after checking the allowed file extensions
    /// </summary> 
    public int AcceptedFilesCount
    {
      get { return this.acceptedFilesCount; }
    }
  }
  #endregion

  #region FileExtensionList class 
  /// <summary>
  /// Collection of FileExtension objects
  /// </summary>    
  [Serializable]
  public class FileExtensionList : List<FileExtension>
  {
    public FileExtensionList()
    {
    }
    /// <summary>
    /// Verify if list of file extensions contains the given extension (as string)
    /// </summary>    
    public bool Contains(string extension)
    {
      List<FileExtension> fileExtensionList = this.FindAll(e => e.Extension == extension.Replace(".", ""));
      return (fileExtensionList.Count > 0);
    }

    /// <summary>
    /// Add FileExtension object by passing extension as string
    /// </summary>    
    public void Add(string extension)
    {
      this.Add(new FileExtension(extension));
    }
    /// <summary>
    /// Add FileExtension object by passing extension as string
    /// </summary>    
    public string GetList()
    {
      if (this.Count == 0)
      {
        return "(all)";
      }
      else
      {
        string listExt = "";
        for (int i = 0; i < this.Count; i++)
        {
          listExt += this[i].Extension + ";";
        }
        return listExt;
      }
    }
  }
  #endregion

  #region FileExtension class
  /// <summary>
  /// Object to store file extension
  /// </summary>  
  public class FileExtension : Object
  {
    public FileExtension()
    {
    }

    public FileExtension(string extension)
    {
      this.extension = extension;
    }

    private string extension;
    /// <summary>
    /// File extension (string, max 4 characters)
    /// </summary>
    [Category("Data")]
    [Description("File extension (string, max 4 characters)")]
    public string Extension
    {
      get { return extension; }
      set { extension = value.Replace(".","").ToUpper(); }
    }
  }
  #endregion

  #region FileDrop class
  /// <summary>
  /// This component makes it easy to drag and drop files from the Windows Explorer to your .NET application.
  /// Just link a control or a form to this FileDrop component and set the list of allowed file extensions.
  /// </summary>
  [ToolboxBitmap(typeof(ResourceFinder), "ScipBe.Controls.WinForms.Resources.FileDrop.ico")]
  [DefaultEvent("FilesDropped")]
  [DefaultProperty("AllowDrop")]
  [Description("Capture dropping files from the Windows Explorer on a form or control")]
  public class FileDrop : Component, IMessageFilter, ISupportInitialize
  {
    public FileDrop()
    {
      allowDrop = true;
      allowedFileExtensions = new FileExtensionList();
      dropControl = null;
      initializing = false;
      if (!this.DesignMode)
      {
        Application.AddMessageFilter(this);
      }
    }

    private bool initializing;

    public void BeginInit()
    {
      initializing = true;
    }

    public void EndInit()
    {
      DragAcceptFilesForControl(dropControl);
      initializing = false;
    }

    /// <summary>
    /// Save handle of control which WM_DROPFILES messages should be accepted
    /// </summary>    
    private System.IntPtr registeredHandle;

    private bool allowDrop;

    /// <summary>
    /// Indicates whether the linked control can accept files that the user drops on it
    /// </summary>  
    /// [Category("Behavior")]
    [Category("Behavior")]
    [Description("Indicates whether the linked control can accept files that the user drops on it")]
    [DesignerSerializationVisibility(DesignerSerializationVisibility.Visible)]
    [MergableProperty(true)]
    [DefaultValue(true)]
    public bool AllowDrop
    {
      get { return allowDrop; }
      set 
      { 
        allowDrop = value;
        if ((!this.DesignMode) && (!initializing))
        {
          DragAcceptFilesForControl(dropControl);
        }
      }
    }

    private FileExtensionList allowedFileExtensions;
    /// <summary>
    /// List of allowed file extensions. When empty, all files will be accepted
    /// </summary> 
    [Category("Behavior")]
    [Description("List of allowed file extensions. When empty, all files will be accepted")]
    [DesignerSerializationVisibility(DesignerSerializationVisibility.Content)]
    [TypeConverter(typeof(AllowedFileExtensionsConverter))]
    public FileExtensionList AllowedFileExtensions
    {
      get { return allowedFileExtensions; }
      set { allowedFileExtensions = value; }
    }
    
    private Control dropControl;
    /// <summary>
    /// The control or form which will accept files when the user drops them on it
    /// </summary>
    [Category("Behavior")]
    [Description("The control or form which will accept files when the user drops them on it")]
    [DesignerSerializationVisibility(DesignerSerializationVisibility.Visible)]
    public Control DropControl
    {
      get
      {
        return dropControl;
      }
      set
      {
        if ((!this.DesignMode) && (!initializing))
        {
          DragAcceptFilesForControl(value);
        }
        
        dropControl = value;
      }
    }

    /// <summary>
    /// Filter Windows messages and process WM_DROPFILES message
    /// </summary>
    public bool PreFilterMessage(ref Message m)
    {
      if ((allowDrop) 
       && (((dropControl != null) && (m.Msg == Win32.WM_DROPFILES))
       && ((m.HWnd == dropControl.Handle) || (dropControl is Form))))
      {
        ProcessDropMessage(m);
        return true;
      }
      else
      {
        return false;
      }
    }

    public delegate void FilesDroppedEventHandler(object source, FilesDroppedEventArgs e);

    /// <summary>
    /// This event is triggered when file(s) is/are dropped on control or form
    /// </summary>
    [Category("Action")]
    [Description("This event is triggered when file(s) is/are dropped on control or form")]
    public event FilesDroppedEventHandler FilesDropped;


    protected bool CheckAcceptFile(string fileName)
    {
      bool allow = false;
      if (allowedFileExtensions.Count > 0)
      {
        // Verify if file is allowed by checking the allowed file extension list
        FileInfo fileInfo = new FileInfo(fileName.ToUpper());
        if (allowedFileExtensions.Contains(fileInfo.Extension))
        {
          allow = true;
        }
      }
      else
      {
        allow = true;
      }
      return allow;
    }

    /// <summary>
    /// Call Win32 DragAcceptFiles function to register current control which will 
    /// accept WM_DROPFILES messages or to unregister old control
    /// </summary>
    protected void DragAcceptFilesForControl(Control control)
    {
      if (registeredHandle.ToInt32() != 0)
      {
        Win32.DragAcceptFiles(registeredHandle, false);
        registeredHandle = control.Handle;
      }
      if ((control != null) && (control.Handle.ToInt32() != 0) && (allowDrop))
      {
        Win32.DragAcceptFiles(control.Handle, true);
        registeredHandle = control.Handle;
      }
    }

    /// <summary>
    /// Process WM_DROPFILES message, verify dropped files and trigger event 
    /// </summary>
    protected void ProcessDropMessage(Message m)
    {
      if (m.Msg == Win32.WM_DROPFILES)
      {
        IntPtr dropHandle = m.WParam;
        int totalFilesCount = Win32.DragQueryFile(dropHandle, -1, null, 0);

        if (totalFilesCount > 0)
        {
          List<string> acceptedFileNames = new List<string>();

          // Run through files
          for (int i = 0; i < totalFilesCount; i++)
          {
            // Get length of the filename
            int fileNameLength = Win32.DragQueryFile(dropHandle, i, null, 0) + 1;
            // Allocate more memory when name is longer then reserved space
            StringBuilder fileName = new StringBuilder(fileNameLength);
            // Get filename
            Win32.DragQueryFile(dropHandle, i, fileName, fileNameLength);

            if (CheckAcceptFile(fileName.ToString()))
            {
              acceptedFileNames.Add(fileName.ToString());  
            }
          }
          // Get coordinates where files have been dropped
          Win32.POINT queryPoint = new Win32.POINT();
          Win32.DragQueryPoint(dropHandle, queryPoint);

          Win32.DragFinish(dropHandle);

          int acceptedFilesCount = acceptedFileNames.Count;
          Point dropPoint = new Point(queryPoint.x, queryPoint.y);

          if (totalFilesCount > 0)
          {
            // Trigger event and pass filenames and coordinates
            FilesDroppedEventArgs e = new FilesDroppedEventArgs(acceptedFileNames, dropPoint, totalFilesCount, acceptedFilesCount);
            if (FilesDropped != null)
            {
              FilesDropped(this, e);
            }
          }
        }
        else
        {
          Win32.DragFinish(dropHandle);
        }
      }
    }
  }
  #endregion
}
